package PLex::Parser;

use PLex;
use PLex::Context;
use strict;
use vars qw($VERSION @ISA);

require AutoLoader;

@ISA = qw(PLex AutoLoader);
$VERSION = '0.01';

# pre-loaded methods
sub new {
    my ($class, $context, $namespace, $lexer) = @_;
    my $self = PLex->new();
    bless $self, $class;

    $self->{'_lexer'} = $lexer;
    $self->{'_context'} = $context;
    $self->{'_namespace'} = $namespace;
    return $self;
}

# AUTOLOAD stubs for inheritance
sub parse;
sub switch_context;
sub eval;
sub context;
sub namespace;
sub lexer;
sub _handle;

# 1;
# __END__

=head1 NAME

PLex::Parser - base parsing module for Perl-Lexer

=head1 SYNOPSIS

  use PLex::Parser;

=head1 DESCRIPTION

The Parser takes a token tree produced by PLex::Lexer and translates
each token in sequence. All of the support code is defined in this
base module. To create a parser, all you have to do is define the stub
method _handle(). The _handle() method should take one argument, a
token. You can add optional arguments as necessary (I think having the
option to hand _handle() a look-ahead and the lex tree is a good way
to handle binary and unary operators if you language requires it, but
I did not want to enforce that in the interface).

The parser should return -1 and set an error code with line and file
if it encounters any tokenizing, syntax or executation errors.

=head1 METHODS

The PLex::Parser object provides the following methods:

=over 4

=item *

new()

I<Usage>

    $parser = new PLex::Parser($context, $namespace, $lexer);

I<Description>

Creates a new Parser for the given context.

=item *

parse()

I<Usage>

    my $tree
    $lexer->lex($script, @$tree) || print $lexer->strerror();
    $parser->parse($tree) || print $parser->strerror();

I<Description>

Parses $tree and executes the requested instructions. Returns -1 on
parsing errors. You will have to defined _handle() before this is useful.

=item *

eval()

I<Usage>

    $self->eval($tree, \$res);

I<Description>

This utility "mini-parser" creates and switches to a new, temporary
context that writes its output to the $res reference. It is useful for
reducing small fragments of code to their final values (pre-processing
function arguments is a good example).

=item *

switch_context()

I<Usage>

    $old_context = $parser->switch_context($context);

I<Description>

Swaps the current context for the given one, preserving the old one
as the return value.

=item *

context(),namespace(),lexer()

I<Description>

returns the various named objects

=back

=head1 AUTHOR

Kyle Downey, kdowney@xline.com

=head1 COPYRIGHT

Copyright 1997, X-Line Software. This software is freely-redistributable
under the terms of the GNU General Public License

=head1 SEE ALSO

PLex(3), PLex::Lexer(3), PLex::Namespace(3), PLex::Context(3).

=cut

sub parse {
    my ($self, $tree) = @_;

    my ($elem, $lookahead);
    if (ref($tree) eq 'ARRAY') {
	while ($elem = shift @$tree) {
	    $lookahead = shift @$tree;
	    unshift @$tree, $lookahead;
	    if ($self->_handle($elem, $lookahead, $tree) == -1) {
		return $self->error("[$elem->{'src'}:$elem->{'line'}]: " . $self->strerror);
	    }
	}
    }
    elsif (ref($tree) eq 'HASH') {
	if ($self->_handle($tree) == -1) {
	    return $self->error("[$elem->{'src'}:$elem->{'line'}]: " . $self->strerror);
	}
    }
    else {
	return $self->error("serious internal lexing error--mangled token tree");
    }
    return 1;
}

sub eval {
    my ($self, $tree, $res) = @_;
    
    my $context = new PLex::Context();
    my $old_context = $self->switch_context($context);
    if ($self->parse($tree) == -1) {
	return -1;
    }
    $self->switch_context($old_context);
    $$res = $context->output(); 
    return 1;
}

sub switch_context {
    my ($self, $context) = @_;
    my $old_context = $self->context();
    $self->{'_context'} = $context;
    return $old_context;
}

sub context {
    return shift->{'_context'};
}

sub namespace {
    return shift->{'_namespace'};
}

sub lexer {
    return shift->{'_lexer'};
}

1;

__END__
